# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['phi', 'phi.old', 'phi.optimus_prime', 'phi.utils']

package_data = \
{'': ['*'], 'phi': ['finetuning/*']}

install_requires = \
['SentencePiece',
 'accelerate',
 'datasets',
 'deepspeed',
 'einops',
 'lion-pytorch',
 'matplotlib',
 'numpy',
 'torch',
 'transformers']

setup_kwargs = {
    'name': 'phi-torch',
    'version': '0.0.3',
    'description': 'andromeda - Pytorch',
    'long_description': '[![Multi-Modality](agorabanner.png)](https://discord.gg/qUtxnK2NMf)\n\nSince Phi is ready to train Agora is actively seeking cloud providers or grant providers to train this all-new revolutionary model and release it open source, if you would like to learn more please email me at `kye@apac.ai`\n\n# Phi: Ultra-Fast and Ultra-Intelligent SOTA Language Model 🚀🌌\n\n[Textbooks Are All You Need](https://arxiv.org/abs/2306.11644)\n\nPhi is a state-of-the-art language model that pushes the boundaries of natural language understanding and generation. Designed for high performance and efficiency, Phi is built upon advanced techniques that make it a strong contender against the likes of OpenAI\'s GPT-4 and PALM.\n\n\n\n# Usage\nGet started:\n\n1. Clone the repository and install the required packages.\n\n\n```\ngit clone https://github.com/kyegomez/Phi\ncd Phi\npip3 install -r requirements.txt\ncd Phi\npython3 training_distributed.py\n```\n\n# Training\n\nFirst:\n\n`Accelerate Config`\n\nEnable Deepspeed 3: \n\n`Accelerate launch train_distributed_accelerate.py`\n\n\n\n## Dataset building building\n\nData\nYou can preprocess a different dataset in a way similar to the C4 dataset used during training by running the build_dataset.py script. This will pre-tokenize, chunk the data in blocks of a specified sequence length, and upload to the Huggingface hub. For example:\n\n```python3 Phi/build_dataset.py --seed 42 --seq_len 8192 --hf_account "HUGGINGFACE APIKEY" --tokenizer "EleutherAI/gpt-neox-20b" --dataset_name "EleutherAI/the_pile_deduplicated"```\n\n\n\n# Inference\n\n```python3 inference.py "My dog is very cute" --seq_len 256 --temperature 0.8 --filter_thres 0.9 --model "phi"``` \n\nNot yet we need to submit model to pytorch hub\n\n\n\n## Model Architecture 🧠🔧\n\n```python\nmodel = TransformerWrapper(\n        num_tokens=64007,\n        max_seq_len=8192,\n        use_abs_pos_emb=False,\n        tokenizer=tokenizer, # !\n        embedding_provider=AndromedaEmbedding(),\n        attn_layers = Decoder(\n            dim=128, # 2048\n            depth=8, # 16\n            dim_head=128,\n            heads=8,\n            alibi_pos_bias=True,\n            alibi_num_heads=4,\n            rotary_xpos=True,\n            attn_flash = True,\n            deepnorm=True,\n            shift_tokens=1,\n            attn_one_kv_head = True,\n            qk_norm=True,\n            attn_qk_norm=True,\n            attn_qk_norm_dim_scale=True # set this to True, in addition to `attn_qk_norm = True`\n        )\n    )\n```\n\n## Roadmap 🗺️📍\n\n1. **Training phase**: Train Phi on a large-scale dataset to achieve SOTA performance in various natural language processing tasks.\n\n2. **World-class inference infrastructure**: Establish a robust and efficient infrastructure that leverages techniques such as:\n\n   - Model quantization: Reduce memory and computational requirements without significant loss in performance.\n   - Distillation: Train smaller, faster models that retain the knowledge of the larger model.\n   - Optimized serving frameworks: Deploy Phi using efficient serving frameworks, such as NVIDIA Triton or TensorFlow Serving, for rapid inference.\n\n3. **Continuous improvement**: Continuously fine-tune Phi on diverse data sources and adapt it to new tasks and domains.\n\n4. **Community-driven development**: Encourage open-source contributions, including pre-processing improvements, advanced training techniques, and novel use cases.\n\n## Why Phi? 🌠💡\n\nPhi can potentially be finetuned with 100k+ token sequence length.\nPhi is a state-of-the-art language model that leverages advanced techniques to optimize its performance and efficiency. Some of these techniques include alibi positional bias, rotary position encodings (xpos), flash attention, and deep normalization (deepnorm). Let\'s explore the benefits of these techniques and provide some usage examples.\n\n### Alibi Positional Bias\n\nAlibi positional bias allows the model to learn relative positions between tokens, enabling it to better capture the relationships and dependencies between tokens in a sequence.\n\nUsage example:\n\n```python\nattn_layers = Decoder(\n    ...\n    alibi_pos_bias=True,\n    alibi_num_heads=4,\n    ...\n)\n```\n\n### Rotary Position Encodings (xpos)\n\nRotary position encodings introduce a more efficient way to encode positions in the input sequence. They avoid the need for absolute positional embeddings, reducing the model\'s memory footprint and improving training speed.\n\nUsage example:\n\n```python\nattn_layers = Decoder(\n    ...\n    rotary_xpos=True,\n    ...\n)\n```\n\n### Flash Attention\n\nFlash attention speeds up the self-attention mechanism by reducing the number of attention computations. It accelerates training and inference while maintaining a high level of performance.\n\nUsage example:\n\n```python\nattn_layers = Decoder(\n    ...\n    attn_flash=True,\n    ...\n)\n```\n\nUsage example:\n\n```python\nattn_layers = Decoder(\n    ...\n    deepnorm=True,\n    ...\n)\n```\n\n### Deep Normalization (deepnorm)\n\nDeep normalization is a technique that normalizes the activations within a layer, helping with training stability and convergence. It allows the model to better learn complex patterns and generalize to unseen data.\n\n# Phi Principles\n- **Efficiency**: Phi incorporates cutting-edge optimization techniques, such as attention flashing, rotary position encodings, and deep normalization, resulting in efficient training and inference.\n\n- **Flexibility**: The modular design of Phi allows for easy adaptation to various tasks and domains, making it a versatile choice for a wide range of applications.\n\n- **Scalability**: Phi\'s architecture is designed to scale with the ever-growing computational resources and data sizes, ensuring its continuous relevance in the NLP landscape.\n\n- **Community-driven**: As an open-source project, Phi thrives on contributions from the community, fostering an environment of collaboration, innovation, and continuous improvement.\n\nJoin us on this exciting journey to create a powerful, efficient, and intelligent language model that will revolutionize the NLP landscape! 🚀🌟\n\n## Todo:\n\n* [Pretrain on Falcon](https://huggingface.co/datasets/tiiuae/falcon-refinedweb)\n\n* [Finetune on this](https://huggingface.co/datasets/Open-Orca/OpenOrca)\n\n* [Create synthetic datasets with the Distiller](https://github.com/Agora-X/The-Distiller)\n\n# Implementing the Phi-1 Model\n\nThis guide is meant to assist you in implementing our Phi-1 model based on the decoder-only transformer model [VSP+ 17] using the FlashAttention implementation of multihead attention (MHA) [DFE+ 22].\n\n## 1. Architecture\n\n1. **Phi-1 model**: Implement an architecture with the following specifications:\n   - 24 layers\n   - Hidden dimension of 2048\n   - MLP-inner dimension of 8192\n   - 32 attention heads of dimension 64 each\n2. **Phi1-small model**: Implement an architecture with the following specifications:\n   - 20 layers\n   - Hidden dimension of 1024\n   - MLP-inner dimension of 4096\n   - 16 attention heads of dimension 64 each\n3. For both architectures, include rotary position embedding [SLP+ 21] with a rotary dimension of 32.\n4. Tokenize your data using the same tokenizer as codegen-350M-mono [NPH+ 22].\n\n## 2. Pretraining\n\n1. Concatenate your dataset into a single dimensional array, using the "⟨∣endoftext∣⟩" token for separating files.\n2. Train your model on a sequence length of 2048 sliced from your dataset array with next-token prediction loss.\n3. Utilize the AdamW optimizer and a linear-warmup-linear-decay learning rate schedule.\n4. Use attention and residual dropout of 0.1.\n5. Execute your training on 8 Nvidia-A100 GPUs using deepspeed.\n6. Use the following specifications for training:\n   - Effective batch size: 1024\n   - Maximum learning rate: 1e-3\n   - Warmup over 750 steps\n   - Weight decay: 0.1\n7. Run your training for a total of 36,000 steps, using the checkpoint at 24,000 steps as your Phi-1-base.\n\n## 3. Finetuning\n\n1. Finetune your Phi-1-base model on your respective finetuning dataset.\n2. Follow the same setup as pretraining, but with different hyperparameters:\n   - Effective batch size: 256\n   - Maximum learning rate: 1e-4\n   - Warmup over 50 steps\n   - Weight decay: 0.01\n3. Run your training for a total of 6,000 steps and pick the best checkpoint (saved every 1000 steps).\n',
    'author': 'Kye Gomez',
    'author_email': 'kye@apac.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kyegomez/phi-1',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
